/**
 * this source file released under the GNU Public Licence.
 * see the accompanying copyright.txt for more information
 * Copyright (C) 2000-2001 Ben Mazur
 */

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

import com.ms.directX.*;

/**
 * PgbDirectDrawVideoOutput is an experimental DirectX (win32 
 * only) output class.  It seems to be one of the few ways to 
 * enlarge the window without slowing down unacceptably.
 * 
 * When compiling with any compiler except Microsoft's,
 * leave this class out.
 */
public final class PgbDirectDrawVideoOutput extends PgbVideoOutput {
	// DirectDraw stuffs
	DirectDraw			dd;
	DirectDrawSurface	ddsPrimary;
	DirectDrawSurface	ddsOffscreen;
	DirectDrawClipper	ddClipper;
	Rect				dest;
	
	// memory
	byte[]				onPixels;
	byte[]				offPixels;
	
	int					rgbByteCount;
	
	public PgbDirectDrawVideoOutput(PgbVideo video) {
		super(video);
		
		// make something up for now...
		rgbByteCount = 1;
		onPixels = new byte[160 * 144 * rgbByteCount];
		offPixels = new byte[160 * 144 * rgbByteCount];
	}
	
	public void reset() {
		;
	}
	
	public void acquire() {
		DDSurfaceDesc 		ddsd;		
		DirectDrawBitmap	bm;
		
		try {
			//
			// create the main DirectDraw object
			//
			dd = new DirectDraw();
			dd.setCooperativeLevel(this, DirectXConstants.DDSCL_NORMAL);

			//
			// Create the primary surface
			//
			ddsd         = new DDSurfaceDesc();
			ddsd.flags   = DirectXConstants.DDSD_CAPS ;
    		ddsd.ddsCaps = DirectXConstants.DDSCAPS_PRIMARYSURFACE;
			ddsPrimary   = dd.createSurface(ddsd); 

			//
			// create a clipper for the primary surface
			//
			ddClipper = dd.createClipper(0);
			ddClipper.setComponent(this);			
			ddsPrimary.setClipper(ddClipper);
			
			//
			// Create the offscreen surface
			//
			ddsd					= new DDSurfaceDesc();
	    	ddsd.flags				= DirectXConstants.DDSD_CAPS | DirectXConstants.DDSD_HEIGHT | DirectXConstants.DDSD_WIDTH;
	    	ddsd.ddsCaps			= DirectXConstants.DDSCAPS_OFFSCREENPLAIN;
	    	ddsd.width				= 160;
	    	ddsd.height				= 144;
			ddsOffscreen			= dd.createSurface(ddsd);
			
			dest			= new Rect();
			dest.left		= 0;
			dest.top		= 0;
			dest.right		= 160;
			dest.bottom		= 144;
			
			//
			// create the two pixel buffers
			//
			DDPixelFormat pf = new DDPixelFormat();
			ddsOffscreen.getPixelFormat(pf);
			rgbByteCount = (pf.rgbBitCount / 8);
			onPixels = new byte[160 * 144 * rgbByteCount];
			offPixels = new byte[160 * 144 * rgbByteCount];
		} catch(Exception ex) {
			// crap... I dunnno...
		}
	}
	
	public void hblank(int line) {
		doLine(offPixels, line);
	}
	
	void doLine(byte[] pixels, int line) {
		int i, offset;
		offset = line * 160;
		if(rgbByteCount == 4) {
			for(i = 0; i < 160; i++) {
				doPixelA8R8G8B8(pixels, (offset + i) * rgbByteCount, video.getScreenMemory(offset + i));
			}
			return;
		}
		if(rgbByteCount == 2) {
			for(i = 0; i < 160; i++) {
				doPixelR5G6B5(pixels, (offset + i) * rgbByteCount, video.getScreenMemory(offset + i));
			}
			return;
		}
	}
	
	void doPixelA8R8G8B8(byte[] pixels, int offset, byte p_index) {
		pixels[offset + 3] = (byte)(0xFF);				//alfa;
		pixels[offset + 2] = video.getScreenRed(p_index);		//red;
		pixels[offset + 1] = video.getScreenGreen(p_index);	//green;
		pixels[offset + 0] = video.getScreenBlue(p_index);	//blu;
	}
	
	void doPixelR5G6B5(byte[] pixels, int offset, byte p_index) {
		pixels[offset + 1] = (byte)((video.getScreenRed(p_index) & 0xF8) | ((video.getScreenGreen(p_index) & 0xE0) >> 5));
		pixels[offset + 0] = (byte)(((video.getScreenGreen(p_index) & 0x0E) << 4) | ((video.getScreenBlue(p_index) & 0xF8) >> 3));
	}
	
	public void vblank() {
		try {
			Rect toScreen;
			DDSurfaceDesc		ddsd = new DDSurfaceDesc();
			// don't need to lock anything, really.
			ddsOffscreen.lock(new Rect(), ddsd, DirectXConstants.DDLOCK_SURFACEMEMORYPTR, 0, onPixels);
				
			// do, however, need to unlock the whole screen
			ddsOffscreen.unlock(dest, ddsd, offPixels);
				
			toScreen 		= new Rect();
			toScreen.left	= 0;
			toScreen.top	= 0;
			toScreen.right	= getSize().width;
			toScreen.bottom = getSize().height;

			ddsPrimary.blt(toScreen, ddsOffscreen, dest, 0);
		} catch(Exception ex) {
			acquire();
		}
	}
}