import java.awt.*;
import java.awt.image.*;

/**
 * PgbDirectVideoOutput uses a java.awt.image.MemoryImageSource
 * with the default RGBColorModel to provide video output.
 */
public class PgbDirectVideoOutput extends PgbVideoOutput {
	MemoryImageSource		screenMISrc;
	Image					screenMImg;
	int[]					screenMemory;

	MemoryImageSource		borderMISrc;
	Image					borderMImg;
	int[]					borderMemory;

	public PgbDirectVideoOutput(PgbVideo video) {
		super(video);
		// color model
		screenMemory = new int[160 * 144];
		screenMISrc = new MemoryImageSource(160, 144, screenMemory, 0, 160);
		screenMISrc.setAnimated(true);
		screenMImg = createImage(screenMISrc);

		borderMemory = new int[256 * 224];
		borderMISrc = new MemoryImageSource(256, 224, borderMemory, 0, 256);
		borderMISrc.setAnimated(true);
		borderMImg = createImage(borderMISrc);
	}
	
	public void reset() {
		;
	}
	
	public void hblank(int line) {
		int i, offset;
		offset = line * 160;
		for(i = 0; i < 160; i++) {
			screenMemory[offset + i] = video.getScreenColor(video.getScreenMemory(line * 160 + i));
		}
	}

	public void vblank() {
		screenMISrc.newPixels();
	}
	
	public void paint(Graphics g) {
		update(g);
	}
	
	public void update(Graphics g) {
		if(PgbSettings.sgbborder) {
			g.drawImage(screenMImg, 48 * PgbSettings.lcdsize, 40 * PgbSettings.lcdsize, 160 * PgbSettings.lcdsize, 144 * PgbSettings.lcdsize, this);
			//g.drawImage(borderMImg, 0, 0, 256 * PgbSettings.lcdsize, 224 * PgbSettings.lcdsize, this);
			// don't trust the border transparency
			g.drawImage(borderMImg, 0 * PgbSettings.lcdsize, 0 * PgbSettings.lcdsize, 256 * PgbSettings.lcdsize, 40 * PgbSettings.lcdsize, 0, 0, 256, 40, this);
			g.drawImage(borderMImg, 0 * PgbSettings.lcdsize, 40 * PgbSettings.lcdsize, 48 * PgbSettings.lcdsize, 184 * PgbSettings.lcdsize, 0, 40, 48, 184, this);
			g.drawImage(borderMImg, 208 * PgbSettings.lcdsize, 40 * PgbSettings.lcdsize, 256 * PgbSettings.lcdsize, 184 * PgbSettings.lcdsize, 208, 40, 256, 184, this);
			g.drawImage(borderMImg, 0 * PgbSettings.lcdsize, 184 * PgbSettings.lcdsize, 256 * PgbSettings.lcdsize, 224 * PgbSettings.lcdsize, 0, 184, 256, 224, this);
		} else {
			g.drawImage(screenMImg, 0, 0, 160 * PgbSettings.lcdsize, 144 * PgbSettings.lcdsize, this);
		}
	}
}
